"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SolanaNtt = void 0;
const anchor_1 = require("@coral-xyz/anchor");
const splToken = __importStar(require("@solana/spl-token"));
const spl_token_1 = require("@solana/spl-token");
const web3_js_1 = require("@solana/web3.js");
const sdk_solana_1 = require("@wormhole-foundation/sdk-solana");
const sdk_solana_core_1 = require("@wormhole-foundation/sdk-solana-core");
const bn_js_1 = __importDefault(require("bn.js"));
const index_js_1 = require("../lib/index.js");
const bindings_js_1 = require("../lib/bindings.js");
class SolanaNtt {
    network;
    chain;
    connection;
    contracts;
    version;
    core;
    pdas;
    program;
    config;
    quoter;
    addressLookupTable;
    constructor(network, chain, connection, contracts, version = "2.0.0") {
        this.network = network;
        this.chain = chain;
        this.connection = connection;
        this.contracts = contracts;
        this.version = version;
        if (!contracts.ntt)
            throw new Error("Ntt contracts not found");
        this.program = (0, bindings_js_1.getNttProgram)(connection, contracts.ntt.manager, version);
        if (this.contracts.ntt?.quoter)
            this.quoter = new index_js_1.NttQuoter(connection, this.contracts.ntt.quoter, this.contracts.ntt.manager);
        this.core = new sdk_solana_core_1.SolanaWormholeCore(network, chain, connection, contracts);
        this.pdas = index_js_1.NTT.pdas(this.program.programId);
    }
    async isRelayingAvailable(destination) {
        if (!this.quoter)
            return false;
        return await this.quoter.isRelayEnabled(destination);
    }
    async quoteDeliveryPrice(destination, options) {
        if (!this.quoter)
            throw new Error("Quoter not available");
        if (!this.quoter.isRelayEnabled(destination))
            throw new Error("Relay not enabled");
        return await this.quoter.quoteDeliveryPrice(destination, options.gasDropoff);
    }
    static async fromRpc(provider, config) {
        const [network, chain] = await sdk_solana_1.SolanaPlatform.chainFromRpc(provider);
        const conf = config[chain];
        if (conf.network !== network)
            throw new Error(`Network mismatch: ${conf.network} != ${network}`);
        if (!("ntt" in conf.contracts))
            throw new Error("Ntt contracts not found");
        const ntt = conf.contracts["ntt"];
        const version = await SolanaNtt.getVersion(provider, 
        //@ts-ignore
        conf.contracts);
        return new SolanaNtt(network, chain, provider, { ...conf.contracts, ntt }, version);
    }
    async getConfig() {
        this.config = this.config ?? (await index_js_1.NTT.getConfig(this.program, this.pdas));
        return this.config;
    }
    async getTokenDecimals() {
        const config = await this.getConfig();
        return await sdk_solana_1.SolanaPlatform.getDecimals(this.chain, this.connection, config.mint);
    }
    async getCustodyAddress() {
        return (await this.getConfig()).custody.toBase58();
    }
    static async getVersion(connection, contracts, sender) {
        return index_js_1.NTT.getVersion(connection, new web3_js_1.PublicKey(contracts.ntt.manager), sender ? new sdk_solana_1.SolanaAddress(sender).unwrap() : undefined);
    }
    async *initialize(sender, args) {
        const mintInfo = await this.connection.getAccountInfo(args.mint);
        if (mintInfo === null)
            throw new Error("Couldn't determine token program. Mint account is null.");
        const payer = new sdk_solana_1.SolanaAddress(sender).unwrap();
        const ix = await index_js_1.NTT.createInitializeInstruction(this.program, {
            ...args,
            payer,
            owner: payer,
            chain: this.chain,
            tokenProgram: mintInfo.owner,
        }, this.pdas);
        const tx = new web3_js_1.Transaction();
        tx.feePayer = payer;
        tx.add(ix);
        yield this.createUnsignedTx({ transaction: tx, signers: [] }, "Ntt.Initialize");
        yield* this.initializeOrUpdateLUT({ payer });
    }
    async *initializeOrUpdateLUT(args) {
        const config = await this.getConfig();
        const ix = await index_js_1.NTT.initializeOrUpdateLUT(this.program, config, {
            payer: args.payer,
            wormholeId: new web3_js_1.PublicKey(this.core.address),
        });
        // Already up to date
        if (!ix)
            return;
        const tx = new web3_js_1.Transaction().add(ix);
        tx.feePayer = args.payer;
        yield this.createUnsignedTx({ transaction: tx }, "Ntt.InitializeLUT");
    }
    async *registerTransceiver(args) {
        const config = await this.getConfig();
        if (config.paused)
            throw new Error("Contract is paused");
        const ix = await this.program.methods
            .registerTransceiver()
            .accountsStrict({
            payer: args.payer.publicKey,
            owner: args.owner.publicKey,
            config: this.pdas.configAccount(),
            transceiver: args.transceiver,
            registeredTransceiver: this.pdas.registeredTransceiver(args.transceiver),
            systemProgram: web3_js_1.SystemProgram.programId,
        })
            .instruction();
        const wormholeMessage = web3_js_1.Keypair.generate();
        const whAccs = sdk_solana_core_1.utils.getWormholeDerivedAccounts(this.program.programId, this.core.address);
        const broadcastIx = await this.program.methods
            .broadcastWormholeId()
            .accountsStrict({
            payer: args.payer.publicKey,
            config: this.pdas.configAccount(),
            mint: config.mint,
            wormholeMessage: wormholeMessage.publicKey,
            emitter: this.pdas.emitterAccount(),
            wormhole: {
                bridge: whAccs.wormholeBridge,
                feeCollector: whAccs.wormholeFeeCollector,
                sequence: whAccs.wormholeSequence,
                program: this.core.address,
                systemProgram: web3_js_1.SystemProgram.programId,
                clock: anchor_1.web3.SYSVAR_CLOCK_PUBKEY,
                rent: anchor_1.web3.SYSVAR_RENT_PUBKEY,
            },
        })
            .instruction();
        const tx = new web3_js_1.Transaction();
        tx.feePayer = args.payer.publicKey;
        tx.add(ix, broadcastIx);
        yield this.createUnsignedTx({ transaction: tx, signers: [wormholeMessage] }, "Ntt.RegisterTransceiver");
    }
    async *setWormholeTransceiverPeer(peer, payer) {
        const sender = new sdk_solana_1.SolanaAddress(payer).unwrap();
        yield this.createUnsignedTx(await index_js_1.NTT.setWormholeTransceiverPeer(this.program, {
            wormholeId: new web3_js_1.PublicKey(this.core.address),
            payer: sender,
            owner: sender,
            chain: peer.chain,
            address: peer.address.toUniversalAddress().toUint8Array(),
        }), "Ntt.SetWormholeTransceiverPeer");
    }
    async *setPeer(peer, tokenDecimals, inboundLimit, payer) {
        const sender = new sdk_solana_1.SolanaAddress(payer).unwrap();
        const ix = await index_js_1.NTT.createSetPeerInstruction(this.program, {
            payer: sender,
            owner: sender,
            chain: peer.chain,
            address: peer.address.toUniversalAddress().toUint8Array(),
            limit: new bn_js_1.default(inboundLimit.toString()),
            tokenDecimals,
        });
        const tx = new web3_js_1.Transaction();
        tx.feePayer = sender;
        tx.add(ix);
        yield this.createUnsignedTx({ transaction: tx }, "Ntt.SetPeer");
    }
    async *transfer(sender, amount, destination, options, outboxItem) {
        const config = await this.getConfig();
        if (config.paused)
            throw new Error("Contract is paused");
        outboxItem = outboxItem ?? web3_js_1.Keypair.generate();
        const payerAddress = new sdk_solana_1.SolanaAddress(sender).unwrap();
        const fromAuthority = payerAddress;
        const from = await this.getTokenAccount(fromAuthority);
        const transferArgs = index_js_1.NTT.transferArgs(amount, destination, options.queue);
        const txArgs = {
            transferArgs,
            payer: payerAddress,
            from,
            fromAuthority,
            outboxItem: outboxItem.publicKey,
        };
        const approveIx = splToken.createApproveInstruction(from, this.pdas.sessionAuthority(fromAuthority, transferArgs), fromAuthority, amount, [], config.tokenProgram);
        const transferIx = config.mode.locking != null
            ? index_js_1.NTT.createTransferLockInstruction(this.program, config, txArgs, this.pdas)
            : index_js_1.NTT.createTransferBurnInstruction(this.program, config, txArgs, this.pdas);
        const releaseIx = index_js_1.NTT.createReleaseOutboundInstruction(this.program, {
            payer: payerAddress,
            outboxItem: outboxItem.publicKey,
            revertOnDelay: !options.queue,
            wormholeId: new web3_js_1.PublicKey(this.core.address),
        }, this.pdas);
        const tx = new web3_js_1.Transaction();
        tx.feePayer = payerAddress;
        tx.add(approveIx, ...(await Promise.all([transferIx, releaseIx])));
        if (options.automatic) {
            if (!this.quoter)
                throw new Error("No quoter available, cannot initiate an automatic transfer.");
            const fee = await this.quoteDeliveryPrice(destination.chain, options);
            const relayIx = await this.quoter.createRequestRelayInstruction(payerAddress, outboxItem.publicKey, destination.chain, Number(fee) / web3_js_1.LAMPORTS_PER_SOL, 
            // Note: quoter expects gas dropoff to be in terms of gwei
            Number(options.gasDropoff ?? 0n) / index_js_1.WEI_PER_GWEI);
            tx.add(relayIx);
        }
        const luts = [];
        try {
            luts.push(await this.getAddressLookupTable());
        }
        catch { }
        const messageV0 = new web3_js_1.TransactionMessage({
            payerKey: payerAddress,
            instructions: tx.instructions,
            recentBlockhash: (await this.connection.getRecentBlockhash()).blockhash,
        }).compileToV0Message(luts);
        const vtx = new web3_js_1.VersionedTransaction(messageV0);
        yield this.createUnsignedTx({ transaction: vtx, signers: [outboxItem] }, "Ntt.Transfer");
    }
    async getTokenAccount(sender) {
        const config = await this.getConfig();
        const tokenAccount = await splToken.getAssociatedTokenAddress(config.mint, sender, true, config.tokenProgram);
        return tokenAccount;
    }
    async *createAta(sender) {
        const config = await this.getConfig();
        const senderAddress = new sdk_solana_1.SolanaAddress(sender).unwrap();
        const ata = await this.getTokenAccount(senderAddress);
        // If the ata doesn't exist yet, create it
        const acctInfo = await this.connection.getAccountInfo(ata);
        if (acctInfo === null) {
            const transaction = new web3_js_1.Transaction().add((0, spl_token_1.createAssociatedTokenAccountInstruction)(senderAddress, ata, senderAddress, config.mint, config.tokenProgram));
            transaction.feePayer = senderAddress;
            yield this.createUnsignedTx({ transaction }, "Redeem.CreateATA");
        }
    }
    async *redeem(attestations, payer) {
        const config = await this.getConfig();
        if (config.paused)
            throw new Error("Contract is paused");
        // TODO: not this, we should iterate over the set of enabled xcvrs?
        // if (attestations.length !== this.xcvrs.length) throw "No";
        const wormholeNTT = attestations[0];
        // Create the vaa if necessary
        yield* this.createAta(payer);
        // Post the VAA that we intend to redeem
        yield* this.core.postVaa(payer, wormholeNTT);
        const senderAddress = new sdk_solana_1.SolanaAddress(payer).unwrap();
        const receiveMessageIx = index_js_1.NTT.createReceiveWormholeMessageInstruction(this.program, {
            wormholeId: new web3_js_1.PublicKey(this.core.address),
            payer: senderAddress,
            vaa: wormholeNTT,
        }, this.pdas);
        const nttMessage = wormholeNTT.payload.nttManagerPayload;
        const emitterChain = wormholeNTT.emitterChain;
        const releaseArgs = {
            payer: senderAddress,
            config,
            nttMessage,
            recipient: new web3_js_1.PublicKey(nttMessage.payload.recipientAddress.toUint8Array()),
            chain: emitterChain,
            revertOnDelay: false,
        };
        const redeemIx = index_js_1.NTT.createRedeemInstruction(this.program, config, {
            payer: senderAddress,
            vaa: wormholeNTT,
        });
        const releaseIx = config.mode.locking != null
            ? index_js_1.NTT.createReleaseInboundUnlockInstruction(this.program, config, releaseArgs)
            : index_js_1.NTT.createReleaseInboundMintInstruction(this.program, config, releaseArgs);
        const tx = new web3_js_1.Transaction();
        tx.feePayer = senderAddress;
        tx.add(...(await Promise.all([receiveMessageIx, redeemIx, releaseIx])));
        const luts = [];
        try {
            luts.push(await this.getAddressLookupTable());
        }
        catch { }
        const messageV0 = new web3_js_1.TransactionMessage({
            payerKey: senderAddress,
            instructions: tx.instructions,
            recentBlockhash: (await this.connection.getRecentBlockhash()).blockhash,
        }).compileToV0Message(luts);
        const vtx = new web3_js_1.VersionedTransaction(messageV0);
        yield this.createUnsignedTx({ transaction: vtx }, "Ntt.Redeem");
    }
    async getCurrentOutboundCapacity() {
        const rl = await this.program.account.outboxRateLimit.fetch(this.pdas.outboxRateLimitAccount());
        return BigInt(rl.rateLimit.capacityAtLastTx.toString());
    }
    async getCurrentInboundCapacity(fromChain) {
        const rl = await this.program.account.inboxRateLimit.fetch(this.pdas.inboxRateLimitAccount(fromChain));
        return BigInt(rl.rateLimit.capacityAtLastTx.toString());
    }
    async getIsExecuted(attestation) {
        if (!this.getIsApproved(attestation))
            return false;
        const { emitterChain } = attestation;
        const inboundQueued = await this.getInboundQueuedTransfer(emitterChain, attestation);
        return inboundQueued === null;
    }
    async getIsApproved(attestation) {
        const digest = attestation.hash;
        const vaaAddress = sdk_solana_core_1.utils.derivePostedVaaKey(this.core.address, Buffer.from(digest));
        try {
            const info = this.connection.getAccountInfo(vaaAddress);
            return info !== null;
        }
        catch (_) { }
        return false;
    }
    async *completeInboundQueuedTransfer(fromChain, transceiverMessage, token, payer) {
        const config = await this.getConfig();
        if (config.paused)
            throw new Error("Contract is paused");
        const senderAddress = new sdk_solana_1.SolanaAddress(payer).unwrap();
        const tx = new web3_js_1.Transaction();
        tx.feePayer = senderAddress;
        const releaseArgs = {
            payer: senderAddress,
            config,
            nttMessage: transceiverMessage,
            recipient: new web3_js_1.PublicKey(transceiverMessage.payload.recipientAddress.toUint8Array()),
            chain: fromChain,
            revertOnDelay: false,
        };
        tx.add(await (config.mode.locking != null
            ? index_js_1.NTT.createReleaseInboundUnlockInstruction(this.program, config, releaseArgs)
            : index_js_1.NTT.createReleaseInboundMintInstruction(this.program, config, releaseArgs)));
        yield this.createUnsignedTx({ transaction: tx }, "Ntt.CompleteInboundTransfer");
    }
    async getInboundQueuedTransfer(chain, nttMessage) {
        const inboxItem = await this.program.account.inboxItem.fetch(this.pdas.inboxItemAccount(chain, nttMessage));
        if (!inboxItem)
            return null;
        const { recipientAddress, amount, releaseStatus } = inboxItem;
        const rateLimitExpiry = releaseStatus.releaseAfter
            ? releaseStatus.releaseAfter[0].toNumber()
            : 0;
        const xfer = {
            recipient: new sdk_solana_1.SolanaAddress(recipientAddress),
            amount: BigInt(amount.toString()),
            rateLimitExpiryTimestamp: rateLimitExpiry,
        };
        return xfer;
    }
    async getAddressLookupTable(useCache = true) {
        if (!useCache || !this.addressLookupTable) {
            const alut = await index_js_1.NTT.getAddressLookupTable(this.program, this.pdas);
            if (alut)
                this.addressLookupTable = alut;
        }
        if (!this.addressLookupTable)
            throw new Error("Address lookup table not found. Did you forget to call initializeLUT?");
        return this.addressLookupTable;
    }
    createUnsignedTx(txReq, description, parallelizable = false) {
        return new sdk_solana_1.SolanaUnsignedTransaction(txReq, this.network, this.chain, description, parallelizable);
    }
}
exports.SolanaNtt = SolanaNtt;
//# sourceMappingURL=ntt.js.map