import { Program } from "@coral-xyz/anchor";
import { AddressLookupTableAccount, Connection, Keypair, PublicKey } from "@solana/web3.js";
import { Chain, Network } from "@wormhole-foundation/sdk-base";
import { AccountAddress, ChainAddress, ChainsConfig, Contracts, TokenAddress, UnsignedTransaction } from "@wormhole-foundation/sdk-definitions";
import { Ntt } from "@wormhole-foundation/sdk-definitions-ntt";
import { SolanaChains, SolanaPlatformType, SolanaTransaction, SolanaUnsignedTransaction } from "@wormhole-foundation/sdk-solana";
import { SolanaWormholeCore } from "@wormhole-foundation/sdk-solana-core";
import { NTT, NttQuoter } from "../lib/index.js";
import { IdlVersion, NttBindings } from "../lib/bindings.js";
export declare class SolanaNtt<N extends Network, C extends SolanaChains> implements Ntt<N, C> {
    readonly network: N;
    readonly chain: C;
    readonly connection: Connection;
    readonly contracts: Contracts & {
        ntt?: Ntt.Contracts;
    };
    readonly version: string;
    core: SolanaWormholeCore<N, C>;
    pdas: NTT.Pdas;
    program: Program<NttBindings.NativeTokenTransfer<IdlVersion>>;
    config?: NttBindings.Config<IdlVersion>;
    quoter?: NttQuoter;
    addressLookupTable?: AddressLookupTableAccount;
    constructor(network: N, chain: C, connection: Connection, contracts: Contracts & {
        ntt?: Ntt.Contracts;
    }, version?: string);
    isRelayingAvailable(destination: Chain): Promise<boolean>;
    quoteDeliveryPrice(destination: Chain, options: Ntt.TransferOptions): Promise<bigint>;
    static fromRpc<N extends Network>(provider: Connection, config: ChainsConfig<N, SolanaPlatformType>): Promise<SolanaNtt<N, SolanaChains>>;
    getConfig(): Promise<NttBindings.Config<IdlVersion>>;
    getTokenDecimals(): Promise<number>;
    getCustodyAddress(): Promise<string>;
    static getVersion(connection: Connection, contracts: Contracts & {
        ntt: Ntt.Contracts;
    }, sender?: AccountAddress<SolanaChains>): Promise<IdlVersion>;
    initialize(sender: AccountAddress<C>, args: {
        mint: PublicKey;
        mode: Ntt.Mode;
        outboundLimit: bigint;
    }): AsyncGenerator<SolanaUnsignedTransaction<N, C>, void, unknown>;
    initializeOrUpdateLUT(args: {
        payer: PublicKey;
    }): AsyncGenerator<SolanaUnsignedTransaction<N, C>, void, unknown>;
    registerTransceiver(args: {
        payer: Keypair;
        owner: Keypair;
        transceiver: PublicKey;
    }): AsyncGenerator<SolanaUnsignedTransaction<N, C>, void, unknown>;
    setWormholeTransceiverPeer(peer: ChainAddress, payer: AccountAddress<C>): AsyncGenerator<SolanaUnsignedTransaction<N, C>, void, unknown>;
    setPeer(peer: ChainAddress, tokenDecimals: number, inboundLimit: bigint, payer: AccountAddress<C>): AsyncGenerator<SolanaUnsignedTransaction<N, C>, void, unknown>;
    transfer(sender: AccountAddress<C>, amount: bigint, destination: ChainAddress, options: Ntt.TransferOptions, outboxItem?: Keypair): AsyncGenerator<UnsignedTransaction<N, C>, any, unknown>;
    private getTokenAccount;
    private createAta;
    redeem(attestations: Ntt.Attestation[], payer: AccountAddress<C>): AsyncGenerator<SolanaUnsignedTransaction<N, C>, void, unknown>;
    getCurrentOutboundCapacity(): Promise<bigint>;
    getCurrentInboundCapacity(fromChain: Chain): Promise<bigint>;
    getIsExecuted(attestation: Ntt.Attestation): Promise<boolean>;
    getIsApproved(attestation: Ntt.Attestation): Promise<boolean>;
    completeInboundQueuedTransfer(fromChain: Chain, transceiverMessage: Ntt.Message, token: TokenAddress<C>, payer: AccountAddress<C>): AsyncGenerator<SolanaUnsignedTransaction<N, C>, void, unknown>;
    getInboundQueuedTransfer(chain: Chain, nttMessage: Ntt.Message): Promise<Ntt.InboundQueuedTransfer<C> | null>;
    getAddressLookupTable(useCache?: boolean): Promise<AddressLookupTableAccount>;
    createUnsignedTx(txReq: SolanaTransaction, description: string, parallelizable?: boolean): SolanaUnsignedTransaction<N, C>;
}
//# sourceMappingURL=ntt.d.ts.map